/* IMPORT */
import Oniguruma from './textmate/oniguruma.js';
import Registry from './textmate/registry.js';
import Renderer from './textmate/renderer.js';
import Tokenizer from './textmate/tokenizer.js';
/* MAIN */
class HighMate {
    /* CONSTRUCTOR */
    constructor(options) {
        /* API */
        this.highlightToAbstract = async (options, highlighter) => {
            const lines = options.code.replace(/^(\r?\n|\r)+|(\r?\n|\r)+$/g, '').split(/\r?\n|\r/g);
            const tokens = [];
            await this.tokenize(lines, options.grammar, options.theme, options.signal, (lineTokens, lineIndex) => {
                tokens[lineIndex] = lineTokens;
            });
            const background = options.background || (await this.registry.loadThemeColorsDefault(options.theme)).background;
            const result = highlighter(lines, tokens, background);
            return result;
        };
        this.highlightToDOM = (options) => {
            return this.highlightToAbstract(options, Renderer.toDOM);
        };
        this.highlightToHighlights = (options) => {
            return this.highlightToAbstract(options, Renderer.toHighlights);
        };
        this.highlightToHTML = (options) => {
            return this.highlightToAbstract(options, Renderer.toHTML);
        };
        this.tokenize = (lines, grammar, theme, signal, onTokens) => {
            return this.tokenizer.tokenize(lines, grammar, theme, signal, onTokens);
        };
        const { getGrammar, getOniguruma, getTheme } = options;
        this.oniguruma = new Oniguruma({ getOniguruma });
        this.registry = new Registry({ getGrammar, getTheme, getOniguruma: () => this.oniguruma.load() });
        this.tokenizer = new Tokenizer(this.registry);
    }
}
/* EXPORT */
export default HighMate;
