/* IMPORT */
import { loadOniguruma, OnigurumaScanner, OnigurumaString } from './vendor.js';
import { isShallowEqual, isString, once } from '../utils.js';
/* MAIN */
//TODO: Explore ditching Oniguruma entirely and instead converting Ruby regexes to native JS regexes
class Oniguruma {
    /* CONSTRUCTOR */
    constructor(options) {
        /* API */
        this.fetch = once(async () => {
            const source = await this.options.getOniguruma();
            if (isString(source)) {
                return await (await fetch(source)).arrayBuffer();
            }
            else {
                return source;
            }
        });
        this.load = once(async () => {
            const buffer = await this.fetch();
            await loadOniguruma(buffer);
            const cache = [];
            return {
                createOnigScanner: (patterns) => {
                    const cacheLv1 = patterns.length;
                    const cacheLv2 = patterns[0] || '';
                    const scannersLv1 = (cache[cacheLv1] || (cache[cacheLv1] = {}));
                    const scannersLv2 = (scannersLv1[cacheLv2] || (scannersLv1[cacheLv2] = []));
                    const cached = scannersLv2.find(([cached]) => isShallowEqual(patterns, cached)); //TODO: Optimize this more maybe
                    if (cached)
                        return cached[1];
                    const scanner = new OnigurumaScanner(patterns);
                    scannersLv2.push([patterns, scanner]);
                    return scanner;
                },
                createOnigString: (content) => {
                    return new OnigurumaString(content);
                }
            };
        });
        this.options = options;
    }
}
/* EXPORT */
export default Oniguruma;
