/* IMPORT */
import { TMRegistry } from './vendor.js';
import { isString, memoize, once } from '../utils.js';
/* MAIN */
class Registry {
    /* CONSTRUCTOR */
    constructor(options) {
        /* API */
        this.fetchGrammar = memoize(async (name) => {
            const source = await this.options.getGrammar(name);
            if (isString(source)) {
                return await (await fetch(source)).json();
            }
            else if (source) {
                return source;
            }
            else {
                console.warn(`Missing grammar: "${name}"`);
            }
        });
        this.fetchTheme = memoize(async (name) => {
            const source = await this.options.getTheme(name);
            if (isString(source)) {
                return await (await fetch(source)).json();
            }
            else if (source) {
                return source;
            }
            else {
                console.warn(`Missing theme: "${name}"`);
            }
        });
        this.loadGrammar = memoize(async (name) => {
            return await this.registry.loadGrammar(name) || undefined;
        });
        this.loadTheme = memoize(async (name) => {
            const theme = await this.fetchTheme(name);
            if (!theme)
                return;
            const themeNormalized = this.normalizeTheme(theme);
            return themeNormalized;
        });
        this.loadThemeColors = memoize(async (name) => {
            const theme = await this.loadTheme(name);
            if (!theme)
                return [];
            this.registry.setTheme(theme);
            const colors = this.registry.getColorMap();
            return colors;
        });
        this.loadThemeColorsDefault = memoize(async (name) => {
            const theme = await this.fetchTheme(name);
            const background = theme?.colors?.['editor.background'] || theme?.colors?.['background'] || '#000000';
            const foreground = theme?.colors?.['editor.foreground'] || theme?.colors?.['foreground'] || '#ffffff';
            return { background, foreground };
        });
        this.loadThemeDecorations = once(() => {
            return ['', '', '', '', 'underline'];
        });
        this.loadThemeStyles = once(() => {
            return ['', 'italic', 'bold', '', ''];
        });
        this.normalizeTheme = memoize(async (theme) => {
            theme || (theme = {
                name: 'Fallback',
                settings: []
            });
            theme.settings || (theme.settings = theme.tokenColors || []);
            theme.settings.unshift?.({
                settings: {
                    background: theme.colors?.['editor.background'] || theme.colors?.['background'] || '#000000',
                    foreground: theme.colors?.['editor.foreground'] || theme.colors?.['foreground'] || '#ffffff'
                }
            });
            return theme;
        });
        this.setTheme = (theme) => {
            if (this.theme === theme)
                return;
            this.theme = theme;
            this.registry.setTheme(theme);
        };
        this.options = options;
        this.registry = new TMRegistry({
            onigLib: options.getOniguruma(),
            loadGrammar: this.fetchGrammar
        });
    }
}
/* EXPORT */
export default Registry;
