/* IMPORT */
import isShallowEqual from 'are-shallow-equal';
import { toKebabCase } from 'kasi';
/* MAIN */
const escapeHTML = (() => {
    const replacementsRe = /[&<>"]/g;
    const replacements = { '&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&quot;', "'": '&#39;' };
    const replaceChar = (char) => {
        return replacements[char];
    };
    const replaceChars = (str) => {
        return str.replace(replacementsRe, replaceChar);
    };
    return replaceChars;
})();
const isString = (value) => {
    return typeof value === 'string';
};
const isUndefined = (value) => {
    return typeof value === 'undefined';
};
const memoize = (fn) => {
    const cache = new Map();
    return (...args) => {
        const id = args[0];
        const resultCached = cache.get(id);
        if (!isUndefined(resultCached) || cache.has(id))
            return resultCached;
        const result = fn.apply(undefined, args);
        cache.set(id, result);
        return result;
    };
};
const noop = () => {
    return;
};
const once = (fn) => {
    let called = false;
    let result;
    return () => {
        if (!called) {
            called = true;
            result = fn();
        }
        return result;
    };
};
/* EXPORT */
export { escapeHTML, isShallowEqual, isString, isUndefined, memoize, noop, once, toKebabCase };
